<?php

namespace Drupal\media_folders\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\taxonomy\Entity\Term;

/**
 * Form builder.
 */
class AddFolderForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'add-folder-form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, mixed $folder = NULL) {
    $form['name'] = [
      '#type' => 'textfield',
      '#required' => TRUE,
      '#title' => $this->t('Name'),
      '#default_value' => '',
      '#maxlength' => 40,
    ];

    $form['description'] = [
      '#type' => 'textarea',
      '#required' => FALSE,
      '#title' => $this->t('Description'),
      '#default_value' => '',
    ];

    $form['parent'] = [
      '#type' => 'hidden',
      '#value' => ($folder) ? $folder->id() : 0,
    ];

    $form['actions'] = [
      '#type' => 'actions',
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
      '#button_type' => 'primary',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    _media_folders_validate_folder_name($form_state, $values['parent'], $values['name']);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $term_values = [
      'parent' => [$values['parent']],
      'name' => $values['name'],
      'description' => $values['description'],
      'vid' => 'media_folders_folder',
    ];

    $term = Term::create($term_values)->save();
    if ($term) {
      $this->messenger()->addStatus('Folder saved');
    }
  }

}
